import argparse
from pathlib import Path
import json
import yaml
from typing import Tuple, Dict, Any

from .registry import ConfigRegistry
from .composite import CompositeConfig
from .helper import ConfigHelper


class ConfigParser:
    def __init__(self):
        self.parser = argparse.ArgumentParser(description="训练脚本")
        
        # 子命令解析器
        self.subparsers = self.parser.add_subparsers(dest="command", help="可用命令")
        
        # 列出可用配置类
        self.list_parser = self.subparsers.add_parser("list", help="列出可用的配置类")
        
        # 查看参数说明
        self.params_parser = self.subparsers.add_parser("params", help="查看配置类的参数说明")
        self.params_parser.add_argument(
            "--type",
            type=str,
            required=True,
            help="配置类型",
        )
        self.params_parser.add_argument(
            "--name",
            type=str,
            required=True,
            help="配置类名称",
        )
        
        # 训练命令
        self.train_parser = self.subparsers.add_parser("train", help="执行训练")
        
        # 动态获取可用的配置类型
        configs = ConfigRegistry.list_available_configs()
        for config_type in configs.keys():
            self.train_parser.add_argument(
                f"--{config_type}_name",
                type=str,
                help=f"{config_type}配置名称",
            )
        self.train_parser.add_argument(
            "--config",
            type=str,
            help="配置文件路径(.json或.yaml)",
        )
        self.train_parser.add_argument(
            "--output_dir",
            type=str,
            default=None,
            help="输出目录",
        )
        self.train_parser.add_argument(
            "--params",
            nargs="*",
            default=[],
            help="其他参数，格式：key=value",
        )
        
        # 初始化配置
        self.configs = None

    def __call__(self):
        """使ConfigParser实例可调用，返回解析后的配置"""
        return self.configs
    
    @staticmethod
    def parse_extra_params(params_list: list = None) -> Dict[str, Any]:
        """解析额外的参数列表
        
        Args:
            params_list: 参数列表，格式为 ["key1=value1", "key2=value2"]
            
        Returns:
            解析后的参数字典
            
        Raises:
            ValueError: 当参数格式错误时抛出
        """
        params = {}
        
        for param in params_list:
            try:
                key, value = param.split("=")
                params[key] = value
            except ValueError:
                raise ValueError(f"参数格式错误: {param}，应为key=value格式")
        
        return params
    
    @staticmethod
    def load_config(config_path: str) -> Dict[str, Any]:
        """从文件加载配置
        
        Args:
            config_path: 配置文件路径
            
        Returns:
            配置对象字典，键为配置类型
            
        Raises:
            ValueError: 当配置文件格式不支持或配置无效时抛出
        """
        path = Path(config_path)
        if not path.exists():
            raise ValueError(f"配置文件不存在: {path}")
        
        # 使用ConfigHelper加载配置文件
        try:
            config = ConfigHelper.load(dict, config_path)
        except (ValueError, TypeError) as e:
            raise ValueError(f"配置文件加载失败: {str(e)}")
        
        if not isinstance(config, dict):
            raise ValueError("配置文件必须是一个字典")
        
        configs = {}
        # 遍历所有配置类型
        for config_type, config_data in config.items():
            if config_type.endswith("_name"):
                # 获取基本配置类型（去掉_name后缀）
                base_type = config_type[:-5]
                if not isinstance(config_data, str):
                    raise ValueError(f"{config_type}的值必须是字符串")
                config_name = config_data
                config_cls = ConfigRegistry.get_config(base_type, config_name)
                
                if config_cls:
                    # 获取对应的配置数据
                    config_data = config.get(f"{base_type}", {})
                    if not isinstance(config_data, dict):
                        raise ValueError(f"{base_type}必须是一个字典")
                    
                    configs[base_type] = ConfigHelper.post_init(config_cls(**config_data))

                    # 如果配置数据中没有name字段，则使用配置名称
                    # 检查配置对象是否有name属性，如果没有或者name为空则使用配置名称
                    if not hasattr(configs[base_type], 'name') or not configs[base_type].name:
                        configs[base_type].name = config[config_type]
        
        return CompositeConfig(**configs)
    
    def parse_args(self, args=None):
        """主解析函数"""
        args = self.parser.parse_args(args)
        configs = None
        
        if args.command == "list":
            # 列出可用的配置类
            configs = ConfigRegistry.list_available_configs()
            print("\n可用的配置类:")
            for config_type, names in configs.items():
                print(f"\n{config_type.title()}配置类:")
                for name in names:
                    print(f"  - {name}")

        elif args.command == "params":
            # 显示参数说明
            try:
                params = ConfigRegistry.get_config_params(args.type, args.name)
                print(f"\n{args.type.title()}配置类 '{args.name}' 的参数说明:")
                for name, doc in params.items():
                    print(f"  {name}: {doc}")
            except ValueError as e:
                print(f"错误: {e}")
                return None

        elif args.command == "train":
            try:
                # 解析额外参数
                params = self.parse_extra_params(args.params)

                # 从配置文件或命令行参数创建配置
                if args.config:
                    configs = self.load_config(args.config)
                else:
                    # 收集所有配置类型和名称
                    config_types = {}
                    for config_type in ConfigRegistry.list_available_configs().keys():
                        name_arg = getattr(args, f"{config_type}_name")
                        if name_arg:
                            config_types[config_type] = name_arg
                    
                    # 验证并分配参数
                    assigned_params = ConfigRegistry.validate_and_assign_params(
                        config_types,
                        params
                    )
                    
                    # 创建配置对象
                    configs = {}
                    for config_type, name in config_types.items():
                        config_cls = ConfigRegistry.get_config(config_type, name)
                        if config_cls:
                            configs[config_type] = ConfigHelper.post_init(config_cls(**assigned_params[config_type]))
                    
                    # 设置输出目录
                    if "training" in configs and args.output_dir:
                        configs["training"].output_dir = args.output_dir
                    
                    # 将配置字典转换为CompositeConfig对象
                    configs = CompositeConfig(**configs)
            except (ValueError, TypeError) as e:
                print(f"错误: {e}")
                return None
        
        return configs


from typing import Dict, Type, Optional, Set, List, Any
import dataclasses
import json
import yaml
from pathlib import Path


class ConfigRegistry:
    """配置注册中心，用于管理不同类型的配置类。

    该类提供了一个统一的配置注册机制，支持注册任意类型的配置类。所有注册的配置类都会被存储在一个统一的字典中，
    并通过配置类型和名称进行索引。同时提供了参数冲突检查、配置验证等功能。

    属性:
        _configs (Dict[str, Dict[str, Type]]): 存储所有注册的配置类

    示例:
        >>> @ConfigRegistry.register("model", "bert")
        >>> @dataclass
        >>> class BertConfig:
        >>>     hidden_size: int = 768
        >>>     num_layers: int = 12
    """
    
    _configs: Dict[str, Dict[str, Type]] = {}
    export_params: Set[str] = set(['name'])
    
    @classmethod
    def register(cls, config_type: str, name: str):
        """注册配置类的装饰器。

        用于将配置类注册到配置中心，支持通过装饰器语法进行注册。注册时会进行参数冲突检查，
        确保不同类型的配置之间不会出现参数名冲突。

        Args:
            config_type (str): 配置类型，如 'model', 'task', 'training' 等
            name (str): 配置名称，用于唯一标识该类型下的配置

        Returns:
            callable: 装饰器函数

        Raises:
            ValueError: 当存在参数冲突时抛出，或当尝试重复注册相同类型和名称的配置类时抛出

        示例:
            >>> @ConfigRegistry.register("model", "bert")
            >>> class BertConfig:
            >>>     hidden_size: int = 768
        """
        def wrapper(config_cls: Type) -> Type:
            if not hasattr(config_cls, "__dataclass_fields__"):
                raise TypeError(f"配置类必须使用@dataclass装饰器定义，得到的是 {config_cls}")
            
            # 检查是否存在重复注册
            if config_type in cls._configs and name in cls._configs[config_type]:
                raise ValueError(f"配置类型 '{config_type}' 下已存在名为 '{name}' 的配置类")
            
            # 检查参数冲突
            cls._check_param_conflicts(config_cls, config_type, name)
            
            # 确保配置类型的字典存在
            if config_type not in cls._configs:
                cls._configs[config_type] = {}
            
            cls._configs[config_type][name] = config_cls
            
            config_cls.name = name
            return config_cls
        return wrapper
    
    @classmethod
    def get_config(cls, config_type: str, name: str) -> Optional[Type]:
        """获取指定类型和名称的配置类。

        Args:
            config_type (str): 配置类型，如 'model', 'task' 等
            name (str): 配置名称

        Returns:
            Optional[Type[]]: 配置类，如果不存在则返回 None

        示例:
            >>> config_cls = ConfigRegistry.get_config("model", "bert")
            >>> config = config_cls(hidden_size=1024)
        """
        return cls._configs.get(config_type, {}).get(name)
    
    @classmethod
    def list_available_configs(cls) -> Dict[str, List[str]]:
        """列出所有可用的配置类。

        Returns:
            Dict[str, List[str]]: 配置类型及其对应的配置名称列表

        示例:
            >>> configs = ConfigRegistry.list_available_configs()
            >>> print(configs)
            {'model': ['bert', 'gpt'], 'task': ['classification']}
        """
        return {config_type: list(configs.keys()) 
                for config_type, configs in cls._configs.items()}
    
    @classmethod
    def get_config_params(cls, config_type: str, name: str) -> Dict[str, str]:
        """获取指定配置类的参数说明。
        
        Args:
            config_type (str): 配置类型
            name (str): 配置名称
            
        Returns:
            Dict[str, str]: 参数名称及其说明的字典

        Raises:
            ValueError: 当指定的配置类不存在时抛出

        示例:
            >>> params = ConfigRegistry.get_config_params("model", "bert")
            >>> print(params)
            {'hidden_size': '隐藏层大小', 'num_layers': '层数'}
        """
        config_cls = cls.get_config(config_type, name)
        if config_cls is None:
            raise ValueError(f"未找到配置：{config_type}/{name}")
        
        return cls._get_param_docs(config_cls)
    
    @classmethod
    def validate_and_assign_params(cls, config_types: Dict[str, str], params: Dict[str, any]) -> Dict[str, Dict[str, any]]:
        """验证并分配参数到相应的配置类。
        
        Args:
            config_types (Dict[str, str]): 配置类型及其名称的映射，如 {"model": "bert", "task": "classification"}
            params (Dict[str, any]): 要分配的参数字典
            
        Returns:
            Dict[str, Dict[str, any]]: 分配后的参数字典，格式为：{"config_name": {"param_name": param_value}}

        Raises:
            ValueError: 当存在无效的配置名称、未分配的参数或缺少必需参数时抛出
        """
        # 获取所有配置类
        config_classes = {}
        for config_type, name in config_types.items():
            config_cls = cls.get_config(config_type, name)
            if config_cls is None:
                raise ValueError(f"Invalid config: {config_type}/{name}")
            config_classes[config_type] = config_cls
        
        # 获取各配置类的参数集合
        config_params = {config_type: set(cls._get_param_names(config_cls))
                        for config_type, config_cls in config_classes.items()}
        
        # 分配参数
        assigned_params = {config_type: {} for config_type in config_types}
        unassigned_params = set()
        
        for param_name, param_value in params.items():
            assigned = False
            for config_type, param_set in config_params.items():
                if param_name in param_set:
                    assigned_params[config_type][param_name] = param_value
                    assigned = True
            
            if not assigned:
                unassigned_params.add(param_name)
        
        # 检查必需参数
        missing_params = []
        for config_type, config_cls in config_classes.items():
            for param_name in cls._get_required_params(config_cls):
                if param_name not in assigned_params[config_type] and param_name not in cls.export_params:
                    missing_params.append(f"{config_type}.{param_name}")
        if missing_params or unassigned_params:
            error_msg = []
            if missing_params:
                error_msg.append(f"缺少必需参数: {', '.join(missing_params)}")
            if unassigned_params:
                error_msg.append(f"未分配的参数: {', '.join(unassigned_params)}")
            raise ValueError("\n".join(error_msg))
        
        return assigned_params
    
    @classmethod
    def _check_param_conflicts(cls, config_cls: Type, config_type: str, name: str) -> None:
        """检查参数冲突。
        
        检查规则：
        1. 不同类型的配置类之间不能有重名参数
        2. 同类型的配置类之间允许有重名参数
    
        Args:
            config_cls (Type): 要检查的配置类
            config_type (str): 配置类型
            name (str): 配置名称
    
        Raises:
            ValueError: 当存在参数冲突时抛出
        """
        new_params = set(cls._get_param_names(config_cls))
        base_params = set(['name'])  # 不再需要获取基类参数
        
        # 检查与其他类型配置的参数冲突
        for other_type, other_configs in cls._configs.items():
            if other_type != config_type:  # 跳过同类型的配置
                for other_name, other_cls in other_configs.items():
                    other_params = set(cls._get_param_names(other_cls))
                    # 排除基类参数后再检查冲突
                    conflicts = (new_params - base_params - cls.export_params) & (other_params - base_params - cls.export_params)
                    if conflicts:
                        raise ValueError(f"参数冲突：{config_type} 配置 '{name}' 与 {other_type} 配置 '{other_name}' 存在重名参数: {conflicts}")
    
    @staticmethod
    def _get_param_names(cls: Type) -> Set[str]:
        """获取类的所有参数名称。

        Args:
            cls (Type): 要获取参数的类

        Returns:
            Set[str]: 参数名称集合
        """
        return {name for name, _ in cls.__dataclass_fields__.items()}
    
    @staticmethod
    def _get_required_params(cls: Type) -> Set[str]:
        """获取类的必需参数名称。

        Args:
            cls (Type): 要获取必需参数的类

        Returns:
            Set[str]: 必需参数名称集合
        """
        return {name for name, field in cls.__dataclass_fields__.items() 
                if field.default == field.default_factory == dataclasses.MISSING}
    
    @staticmethod
    def _get_param_docs(cls: Type) -> Dict[str, str]:
        """获取类的参数文档。

        Args:
            cls (Type): 要获取参数文档的类

        Returns:
            Dict[str, str]: 参数名称及其说明的字典
        """
        return {name: field.metadata.get('help', '') 
                for name, field in cls.__dataclass_fields__.items()}


from dataclasses import dataclass
from typing import Dict, Any
from pathlib import Path
import json
import yaml
from .helper import ConfigHelper


@dataclass
class CompositeConfig:
    
    """组合配置类，用于封装多个子配置。

    提供了通过属性访问子配置的功能，
    并为每个子配置类型添加对应的type属性。

    示例:
        >>> configs = CompositeConfig()
        >>> configs.model.hidden_size  # 访问模型配置的参数
        >>> configs.model_type  # 获取模型类型
    """
    name: str = "composite"
    
    def __init__(self, **configs):
        """初始化组合配置类。
    
        Args:
            **configs: 子配置对象字典，键为配置类型
        """
        self._configs = configs
    
        # 为每个配置类型添加type属性
        for config_type, config in configs.items():
            if hasattr(config, "name"):
                setattr(self, f"{config_type}_name", config.name)

    def __getattr__(self, name: str) -> Any:
        """通过属性访问子配置。

        Args:
            name: 属性名称

        Returns:
            子配置对象或属性值

        Raises:
            AttributeError: 当属性不存在时抛出
        """
        if name in self._configs:
            return self._configs[name]
        raise AttributeError(f"'{self.__class__.__name__}' 对象没有属性 '{name}'")

    def to_dict(self) -> Dict[str, Any]:
        """将配置对象转换为字典格式。

        Returns:
            Dict[str, Any]: 包含所有配置参数的字典
        """
        result = {}
        # 添加子配置
        for config_type, config in self._configs.items():
            result[config_type] = ConfigHelper.to_dict(config)
            if hasattr(config, "name"):
                result[f"{config_type}_name"] = config.name
        
        # 过滤掉以_开头的键
        filtered_result = {}
        for key, value in result.items():
            if key.startswith('_'):
                continue
            if isinstance(value, dict):
                # 过滤嵌套字典中以_开头的键
                filtered_dict = {}
                for sub_key, sub_value in value.items():
                    if not sub_key.startswith('_'):
                        filtered_dict[sub_key] = sub_value
                if filtered_dict:
                    filtered_result[key] = filtered_dict
            else:
                filtered_result[key] = value
                
        return filtered_result
    
    def __str__(self):
        """重写__str__方法，使其返回配置的字典形式的字符串表示"""
        config_dict = self.to_dict()
        # 格式化输出配置字典
        lines = []
        for key, value in config_dict.items():
            if isinstance(value, dict):
                # 处理嵌套字典，增加缩进
                sub_lines = [f"{key}:"]
                for sub_key, sub_value in value.items():
                    sub_lines.append(f"    {sub_key}: {sub_value}")
                lines.extend(sub_lines)
            else:
                lines.append(f"{key}: {value}")
        return "\n".join(lines)

    def __repr__(self):
        """重写__repr__方法，使其返回配置的字典形式的字符串表示"""
        return self.__str__()
    
    def save(self, path: str) -> None:
        """将配置保存到文件。

        支持保存为 JSON 或 YAML 格式，根据文件扩展名自动选择格式。

        Args:
            path (str): 保存配置的文件路径，支持 .json、.yaml 或 .yml 扩展名

        Raises:
            ValueError: 当文件格式不支持时抛出
        """
        ConfigHelper.save(self, path)



import unittest
from pathlib import Path
import json
import yaml
from dataclasses import dataclass, field
from config_manager import ConfigRegistry, ConfigParser

@ConfigRegistry.register("model", "test_model")
@dataclass
class TestModelConfig:
    input_dim: int
    output_dim: int
    hidden_dims: list = field(default_factory=lambda: [256, 128])

@ConfigRegistry.register("training", "test_training")
@dataclass
class TestTrainingConfig:
    learning_rate: float
    batch_size: int
    epochs: int = 10
    output_dir: str = "./output"

class TestConfigParser(unittest.TestCase):
    def setUp(self):
        self.parser = ConfigParser()
        self.test_config_path = Path("test_config.yaml")
    
    def tearDown(self):
        if self.test_config_path.exists():
            self.test_config_path.unlink()
    
    def test_list_command(self):
        """测试list命令，验证是否能正确列出所有可用配置"""
        args = self.parser.parse_args(["list"])
        self.assertIsNotNone(args)
        configs = args
        self.assertIsInstance(configs, dict)
        self.assertIn("model", configs)
        self.assertIn("training", configs)
        self.assertIn("test_model", configs["model"])
        self.assertIn("test_training", configs["training"])
        
        # 打印配置列表，方便观察
        print("\n=== Available Configurations ===")
        for config_type, names in configs.items():
            print(f"{config_type}:")
            for name in names:
                print(f"  - {name}")
        print("============================\n")
    
    def test_params_command(self):
        """测试params命令，验证是否能正确显示配置参数说明"""
        print("\n=== Parameters for test_model ===")
        args = self.parser.parse_args(["params", "--type", "model", "--name", "test_model"])
        self.assertIsNone(args)  # params命令直接打印信息，返回None
        print("==============================\n")
        
        print("\n=== Testing Invalid Config Type ===")
        # 测试无效的配置类型
        args = self.parser.parse_args(["params", "--type", "invalid", "--name", "test_model"])
        self.assertIsNone(args)
        print("================================\n")
    
    def test_train_command_with_config_file(self):
        """测试使用配置文件的train命令"""
        # 创建测试配置文件
        config_data = {
            "model_name": "test_model",
            "model": {
                "input_dim": 10,
                "output_dim": 2
            },
            "training_name": "test_training",
            "training": {
                "learning_rate": 0.001,
                "batch_size": 32
            }
        }
        with open(self.test_config_path, "w") as f:
            yaml.dump(config_data, f)
        
        args = self.parser.parse_args(["train", "--config", str(self.test_config_path)])
        self.assertIsNotNone(args)
        self.assertTrue(hasattr(args, "model"))
        self.assertTrue(hasattr(args, "training"))
        
        # 验证配置内容
        self.assertEqual(args.model.input_dim, 10)
        self.assertEqual(args.model.output_dim, 2)
        self.assertEqual(args.training.learning_rate, 0.001)
        self.assertEqual(args.training.batch_size, 32)
    
    def test_train_command_with_cli_params(self):
        """测试使用命令行参数的train命令"""
        args = self.parser.parse_args([
            "train",
            "--model_name", "test_model",
            "--training_name", "test_training",
            "--params",
            "input_dim=10",
            "output_dim=2",
            "learning_rate=0.001",
            "batch_size=32"
        ])
        
        self.assertIsNotNone(args)
        self.assertTrue(hasattr(args, "model"))
        self.assertTrue(hasattr(args, "training"))
        
        # 验证配置内容
        self.assertEqual(args.model.input_dim, 10)
        self.assertEqual(args.model.output_dim, 2)
        self.assertEqual(args.training.learning_rate, 0.001)
        self.assertEqual(args.training.batch_size, 32)
    
    def test_train_command_with_output_dir(self):
        """测试train命令的输出目录参数"""
        args = self.parser.parse_args([
            "train",
            "--model_name", "test_model",
            "--training_name", "test_training",
            "--output_dir", "./custom_output",
            "--params",
            "input_dim=10",
            "output_dim=2",
            "learning_rate=0.001",
            "batch_size=32"
        ])
        
        self.assertIsNotNone(args)
        self.assertEqual(args.training.output_dir, "./custom_output")
    
    def test_invalid_config_file(self):
        """测试无效的配置文件"""
        # 创建无效的配置文件
        with open(self.test_config_path, "w") as f:
            f.write("invalid: yaml: content:")
        
        with self.assertRaises(yaml.YAMLError):
            self.parser.parse_args(["train", "--config", str(self.test_config_path)])
    
    def test_invalid_params_format(self):
        """测试无效的参数格式"""
        args = self.parser.parse_args([
            "train",
            "--model_name", "test_model",
            "--training_name", "test_training",
            "--params",
            "invalid_param"  # 缺少=号
        ])
        
        self.assertIsNone(args)

if __name__ == "__main__":
    unittest.main()


from typing import Dict, Type, Any
import json
import yaml
from pathlib import Path


class ConfigHelper:
    """配置工具类，提供配置对象的通用功能。
    
    该类提供了配置对象的序列化、反序列化、类型转换等功能，
    可以在registry.py和parser.py中使用这些功能来增强配置类的能力。
    """
    
    @staticmethod
    def to_dict(config_obj) -> Dict[str, Any]:
        """将配置对象转换为字典格式。

        Args:
            config_obj: 配置对象实例

        Returns:
            Dict[str, Any]: 包含所有配置参数的字典
        """
        result = {}
        # 如果name属性存在，则添加到结果中
        if hasattr(config_obj, 'name'):
            result['name'] = config_obj.name
        # 更新其他属性
        result.update({k: v for k, v in config_obj.__dict__.items()})
        return result
    
    @staticmethod
    def save(config_obj, path: str) -> None:
        """将配置保存到文件。

        支持保存为 JSON 或 YAML 格式，根据文件扩展名自动选择格式。

        Args:
            config_obj: 配置对象实例
            path (str): 保存配置的文件路径，支持 .json、.yaml 或 .yml 扩展名

        Raises:
            ValueError: 当文件格式不支持时抛出
        """
        path = Path(path)
        if path.suffix == '.json':
            with open(path, 'w', encoding='utf-8') as f:
                json.dump(ConfigHelper.to_dict(config_obj), f, indent=2, ensure_ascii=False)
        elif path.suffix in {'.yaml', '.yml'}:
            with open(path, 'w', encoding='utf-8') as f:
                yaml.dump(ConfigHelper.to_dict(config_obj), f, allow_unicode=True)
        else:
            raise ValueError(f"不支持的文件格式：{path.suffix}")
    
    @staticmethod
    def load(config_cls: Type, path: str):
        """从文件加载配置。

        支持从 JSON 或 YAML 文件加载配置，根据文件扩展名自动选择格式。

        Args:
            config_cls (Type): 配置类
            path (str): 配置文件的路径，支持 .json、.yaml 或 .yml 扩展名

        Returns:
            配置对象实例

        Raises:
            ValueError: 当文件格式不支持时抛出
        """
        path = Path(path)
        if path.suffix == '.json':
            with open(path, 'r', encoding='utf-8') as f:
                data = json.load(f)
        elif path.suffix in {'.yaml', '.yml'}:
            with open(path, 'r', encoding='utf-8') as f:
                data = yaml.safe_load(f)
        else:
            raise ValueError(f"不支持的文件格式：{path.suffix}")
        return config_cls(**data)
    
    @staticmethod
    def post_init(config_obj):
        """配置对象初始化后的处理函数。

        执行参数的自动类型转换和验证，支持以下功能：
        1. 基本类型（int、float、bool、str）的转换
        2. 字符串到布尔值的转换（'true', 'yes', '1' 转换为 True）
        3. 逗号分隔的字符串到列表的转换
        4. JSON 字符串到字典的转换

        Args:
            config_obj: 配置对象实例

        Raises:
            ValueError: 当参数值无法转换为目标类型时抛出
        """
        for field_name, field in config_obj.__class__.__dataclass_fields__.items():
            value = getattr(config_obj, field_name)
            if value is not None:
                try:
                    # 处理基本类型
                    if field.type in {int, float, bool, str}:
                        if field.type == bool and isinstance(value, str):
                            value = value.lower() in {"true", "yes", "1"}
                        else:
                            value = field.type(value)
                    # 处理列表类型
                    elif field.type == list or (hasattr(field.type, "__origin__") and field.type.__origin__ is list):
                        if isinstance(value, str):
                            try:
                                # 尝试解析为JSON格式
                                items = json.loads(value)
                                if not isinstance(items, list):
                                    # 如果解析成功但不是列表，则按照逗号分隔处理
                                    items = value.split(",")
                            except json.JSONDecodeError:
                                # JSON解析失败，按照逗号分隔处理
                                items = value.split(",")
                            
                            # 如果是泛型列表，获取元素类型；否则使用str类型
                            item_type = field.type.__args__[0] if hasattr(field.type, "__args__") else str
                            # 如果类型是Any，直接使用原值
                            if item_type == Any:
                                value = [item.strip() if isinstance(item, str) else item for item in items]
                            else:
                                value = [item_type(item.strip() if isinstance(item, str) else item) for item in items]
                        elif isinstance(value, (list, tuple)):
                            # 如果已经是列表或元组类型，确保元素类型正确
                            item_type = field.type.__args__[0] if hasattr(field.type, "__args__") else Any
                            # 如果类型是Any，直接使用原值
                            if item_type == Any:
                                value = list(value)
                            else:
                                value = [item_type(item) for item in value]
                    # 处理字典类型
                    elif field.type == dict or (hasattr(field.type, "__origin__") and field.type.__origin__ is dict):
                        if isinstance(value, str):
                            # 假设值是JSON格式的字符串
                            value = json.loads(value)
                        elif isinstance(value, dict):
                            # 如果已经是字典类型，确保键值类型正确
                            if hasattr(field.type, "__args__"):
                                key_type, value_type = field.type.__args__
                                # 如果类型是Any，直接使用原值
                                if key_type == Any:
                                    key_converter = lambda x: x
                                else:
                                    key_converter = key_type
                                if value_type == Any:
                                    value_converter = lambda x: x
                                else:
                                    value_converter = value_type
                                value = {key_converter(k): value_converter(v) for k, v in value.items()}
                    
                    setattr(config_obj, field_name, value)
                except (ValueError, TypeError, json.JSONDecodeError) as e:
                    raise ValueError(f"参数 {field_name} 的值 {value} 无法转换为类型 {field.type}: {str(e)}")
                
        return config_obj


from setuptools import setup, find_packages

setup(
    name="modern-config-manager",
    version="0.2.0",
    description="一个强大而灵活的配置管理系统",
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    author="Kang Chen",
    author_email="chenkangcs@foxmail.com",
    url="https://github.com/queekye/config_manager",
    packages=find_packages(include=['config_manager', 'config_manager.*']),
    install_requires=[
        'dataclasses;python_version<"3.7"',
        'pyyaml>=5.1',
    ],
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
    ],
)